import os.path

global driverPath
global outIdx

outIdx = {"out_0": 0, "out_1": 1, "out_2": 2, "out_3": 3}

driverPath = "/sys/devices/platform/fe5a0000.i2c/i2c-1/1-001b"

def ReadLedValue(leds, path):
    value = {}
    for led in leds:
        try:
            file = open(os.path.join(driverPath, led, path), "r")
            value[led] = file.readline()
            file.close()
        except Exception as e:
            print("\tError: " + str(e))
            return None

    return value


def WriteCommand(cmd, path):
    print("\tWrite command: {0} = {1}".format(path, cmd))
    try:
        file = open(os.path.join(driverPath, path), "w")
        file.write(cmd)
        file.close()
    except Exception as e:
        print("\tError: " + str(e))

def WriteLedsCommand(cmd, leds, path):   
    for led in leds:
        print("\tWrite led command: {0}/{1} = {2}".format(led, path, cmd))
        try:
            file = open(os.path.join(driverPath, led, path), "w")
            file.write(cmd)
            file.close()
        except Exception as e:
            print("\tError: " + str(e))

def ValidateLeds(input, src):
    for led in input:
        if not led in src:
            return False
    return True

def ShowLedConfig(leds):
    # Enable led
    ch = input("Enable led [Y/n]: ")
    enableLed = False
    if ch == "" or ch.lower() =="y":
        enableLed = True
        WriteLedsCommand("1", leds, "enable")
    else:
        WriteLedsCommand("0", leds, "enable")

    # Select mode
    ch = input("Select Mode [Auto/Manual]: ")
    autoMode = False
    if ch == "" or ch.lower() =="auto":
        autoMode = True
        WriteLedsCommand("autonomous", leds, "mode")
    else:
        WriteLedsCommand("manual", leds, "mode")

    PwmDimming = input("PWM dimming scale [1. Linear/2. Exponential]: ").lower()
    if PwmDimming == "" or PwmDimming == "1":
        WriteLedsCommand("linear", leds, "pwm_dimming_scale")
        PwmDimming = "Linear"
    else:
        WriteLedsCommand("exponential", leds, "pwm_dimming_scale")

    PwmAlignment = input("PWM alignment schema [1. Forward Align/2. Middle Align/3. Backward Align]: ").lower()
    if PwmAlignment == "" or PwmAlignment == "1":
        WriteLedsCommand("forward", leds, "pwm_phase_align")
        PwmAlignment = "forward"  
    elif PwmAlignment == "2":
        WriteLedsCommand("middle", leds, "pwm_phase_align")
        PwmAlignment = "middle"  
    else:
        WriteLedsCommand("backward", leds, "pwm_phase_align")
        PwmAlignment = "backward"  

    return  {"Enable": enableLed, "AutoMode": autoMode, "PwmAlignment" : PwmAlignment}
      
def ShowAEUConfig(leds):
    AEUNumber = int(input("AEU Number Selection (1-3) [1.only use AEU1/2. use AEU1 and AEU2/3. use AEU1, AEU2 and AEU3]: ").lower())


    print("Start pause time")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")

    StartPauseTime = input("Select 0-15: ")
    
    print("Stop pause time")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")

    StopPauseTime = input("Select 0-15: ")    

    print("Playback time")
    print("0. 0 times")
    print("1. 1 times")
    print("2. 2 times")
    print("3. 3 times")
    print("4. 4 times")
    print("5. 5 times")
    print("6. 6 times")
    print("7. 7 times")
    print("8. 8 times")
    print("9. 9 times")
    print("10. 10 times")
    print("11. 11 times")
    print("12. 12 times")
    print("13. 13 times")
    print("14. 14 times")
    print("15. intinite times")

    PlaybackTime = input("Select 0-15: ")    


    WriteLedsCommand("{0}:{1}:{2}:{3}".format(AEUNumber, StartPauseTime, StopPauseTime, PlaybackTime), leds, "autonomous_animation")
    return {"AEUNumber": AEUNumber, "StartPauseTime": StartPauseTime, "StopPauseTime" : StopPauseTime, "PlaybackTime":PlaybackTime}   


def ShowAEUxConfig(idx, leds):
    print("AUE{0} Config".format(idx))
    PlaybackTime = input("Playback time (0-3) [0. 0 time/1. 1 time/2. 2 times/3. Infinite times]")

    WriteLedsCommand(PlaybackTime, leds, "AEU{0}/playback_time".format(idx))

    PWM1 = input("PWM1 (0-255):")
    WriteLedsCommand(PWM1, leds, "AEU{0}/pwm1".format(idx))

    PWM2 = input("PWM2 (0-255):")
    WriteLedsCommand(PWM2, leds, "AEU{0}/pwm2".format(idx))
    PWM3 = input("PWM3 (0-255):")
    WriteLedsCommand(PWM3, leds, "AEU{0}/pwm3".format(idx))
    PWM4 = input("PWM4 (0-255):")
    WriteLedsCommand(PWM4, leds, "AEU{0}/pwm4".format(idx))
    PWM5 = input("PWM5 (0-255):")
    WriteLedsCommand(PWM5, leds, "AEU{0}/pwm5".format(idx))

    print("T1")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")
    T1 = input("Select 0-15: ")    
    WriteLedsCommand(T1, leds, "AEU{0}/slope_time_t1".format(idx))

    print("T2")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")
    T2 = input("Select 0-15: ")    
    WriteLedsCommand(T2, leds, "AEU{0}/slope_time_t2".format(idx))

    print("T3")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")
    T3 = input("Select 0-15: ")    
    WriteLedsCommand(T3, leds, "AEU{0}/slope_time_t3".format(idx))

    print("T4")
    print("0. no pause time")
    print("1. 0.09s")
    print("2. 0.18s")
    print("3. 0.36s")
    print("4. 0.54s")
    print("5. 0.80s")
    print("6. 1.07s")
    print("7. 1.52s")
    print("8. 2.06s")
    print("9. 2.50s")
    print("10. 3.04s")
    print("11. 4.02s")
    print("12. 5.01s")
    print("13. 5.99s")
    print("14. 7.06s")
    print("15. 8.05s")
    T4 = input("Select 0-15: ")    
    WriteLedsCommand(T4, leds, "AEU{0}/slope_time_t4".format(idx))


def PerformDriveLeds(leds):
    ledConfig = ShowLedConfig(leds)

    if ledConfig["AutoMode"]:
        DC = input("Auto Dot Current [0-255]: ").lower()
        WriteLedsCommand((DC), leds, "autonomous_dc")
        AEUConfig = ShowAEUConfig(leds)

        for idx in range(1, AEUConfig["AEUNumber"] + 1):
            ShowAEUxConfig(idx, leds)

        while True:
            # Print summary
            print("1. Start")
            print("2. Pause")
            print("3. Continue")
            print("4. Stop")
            print("5. Update command")
            print("6. Reload")
            print("7. Back to main menu")  

            ch = input("Select 1-7:")
            if ch == "1":
                WriteCommand("start", "lp5812_chip_setup/device_command")
            elif ch == "2":
                WriteCommand("pause", "lp5812_chip_setup/device_command")
            elif ch == "3":
                WriteCommand("continue", "lp5812_chip_setup/device_command")
            elif ch == "4":
                WriteCommand("stop", "lp5812_chip_setup/device_command")
            elif ch == "5":
                WriteCommand("update", "lp5812_chip_setup/device_command")
            elif ch == "6":
                return True
            elif ch == "7":
                return False

    else:
        while True:
            DC = input("Dot current [0-255]: ").lower()
            WriteLedsCommand((DC), leds, "manual_dc")
            PWM = input("Manual PWM [0-255]: ").lower()
            WriteLedsCommand((PWM), leds, "manual_pwm")

            while True:
                print("1. Input again")
                print("2. Update command")
                print("3. Read Lod_lsd value")
                print("4. Reset fault")
                print("5. Reload")
                print("6. Back to main menu")                
                ch = input("Select 1-6:")
                if ch == "5":
                    return True
                elif ch == "6":
                    return False
                elif ch == "2":
                    WriteCommand("update", "lp5812_chip_setup/device_command")
                elif ch == "3":
                    values = ReadLedValue(leds, "lod_lsd")
                    if values != None:
                        for led in leds:
                            print("\t {0}.lod_lsd = {1}".format(led, values[led]))
                    input("Press ENTER to continue")
                elif ch == "4":
                    ResetFault()
                else:
                    break


def ShowDirectDriveMode():
    print("\033c", end='')
    print("Direct Drive Mode ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36, J1.OUT0 - J1.OUT3")


def DirectDriveMode():
    WriteCommand("direct_mode", "lp5812_chip_setup/dev_config")
    while True:
        ShowDirectDriveMode()
        ch = input("Select leds (ex: led_1, led_2) [led_0|led_1|led_2|led_3/b (Back to main menu)]: ")
        if ch == "b":
            return

        if ch.strip() == "":
            input("Wrong selected led. Press ENTER to continue")
            continue

        led_input = ch.split(",")
        leds = []
        for led in led_input:
            led = led.strip()
            leds.append(led)
            if ValidateLeds(leds, ["led_0", "led_1", "led_2", "led_3"]) == False:
                input("Wrong selected led. Press ENTER to continue")
                continue
        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return
                   


def ShowScanDriveModeSelectLed(scan_order):
    leds_mapping = {"out_0": "led_A0 | led_A1 | led_A2",
                    "out_1": "led_B0 | led_B1 | led_B2",
                    "out_2": "led_C0 | led_C1 | led_C2",
                    "out_3": "led_D0 | led_D1 | led_D2",
                    }
    
    while True:
        ch = input("Select leds (ex: led_x1, led_x2) [{0}/b (Back to main menu)]: ".format(leds_mapping[scan_order]))
        if ch == "b":
            return None
            
        if ch.strip() == "":
            input("Wrong selected led. Press ENTER to continue")
            continue
        
        led_input = ch.split(",")
        leds = []
        for led in led_input:
            led = led.strip()
            leds.append(led)

        if ValidateLeds(leds, leds_mapping[scan_order]) == False:
            input("Wrong selected led. Press ENTER to continue")
            continue

        return leds


def ShowScanDriveModeWith1Scan():
    print("\033c", end='')
    print("Scan Drive Mode with 1 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36, J5.OUT0 - J0.OUT3")


def ScanDriveModeWith1Scan():
    while True:
        ShowScanDriveModeWith1Scan()
        scan_order = input("Scan order 0 [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if scan_order == "b":
            return

        if not scan_order in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        if ShowJumpOfOut(scan_order) == False:
            continue

        input("Check Jump and press Enter to continue")

        WriteCommand("tcmscan:1:{0}".format(outIdx[scan_order]), "lp5812_chip_setup/dev_config")


        leds = ShowScanDriveModeSelectLed(scan_order)
        if leds == None:
            return

        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return

def ShowScanDriveModeWith2Scan():
    print("\033c", end='')
    print("Scan Drive Mode with 2 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36, J5.OUT0 - J0.OUT3")

def ScanDriveModeWith2Scan():
    while True:
        ShowScanDriveModeWith2Scan()
        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")
        if scan_order_0 == "b":
            return

        if not scan_order_0 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        scan_order_1 = input("Scan order 1 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_1 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue


        if ShowJumpOfOut(scan_order_0) == False:
            continue

        if scan_order_1 != scan_order_0:
            if ShowJumpOfOut(scan_order_1) == False:
                continue

        input("Check Jump and press Enter to continue")

        WriteCommand("tcmscan:2:{0}:{1}".format(outIdx[scan_order_0], outIdx[scan_order_1]), "lp5812_chip_setup/dev_config")

        leds = ShowScanDriveModeSelectLed(scan_order_0)
        if leds == None:
            return
        
        if scan_order_1 != scan_order_0:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_1)

        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    

def ShowScanDriveModeWith3Scan():
    print("\033c", end='')
    print("Scan Drive Mode with 3 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36, J5.OUT0 - J0.OUT3")

def ScanDriveModeWith3Scan():
    while True:
        ShowScanDriveModeWith3Scan()
        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if scan_order_0 == "b":
            return

        scan_order_1 = input("Scan order 1 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_1 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue
        scan_order_2 = input("Scan order 2 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_2 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue




        if ShowJumpOfOut(scan_order_0) == False:
            continue

        if scan_order_1 != scan_order_0:
            if ShowJumpOfOut(scan_order_1) == False:
                continue


        if scan_order_2 != scan_order_0 and scan_order_1 != scan_order_2:
            if ShowJumpOfOut(scan_order_2) == False:
                continue

        input("Check Jump and press Enter to continue")

        WriteCommand("tcmscan:3:{0}:{1}:{2}".format(outIdx[scan_order_0], outIdx[scan_order_1], outIdx[scan_order_2]), "lp5812_chip_setup/dev_config")

        leds = ShowScanDriveModeSelectLed(scan_order_0)
        if leds == None:
            return
        
        if scan_order_1 != scan_order_0:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_1)


        if scan_order_2 != scan_order_0 and scan_order_2 != scan_order_1:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_2)

        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    


def ShowScanDriveModeWith4Scan():
    print("\033c", end='')
    print("Scan Drive Mode with 3 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36, J5.OUT0 - J0.OUT3")

def ScanDriveModeWith4Scan():
    while True:
        ShowScanDriveModeWith3Scan()
        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if scan_order_0 == "b":
            return

        scan_order_1 = input("Scan order 1 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_1 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue
        scan_order_2 = input("Scan order 2 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_2 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue
        scan_order_3 = input("Scan order 2 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_3 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue


        if ShowJumpOfOut(scan_order_0) == False:
            continue

        if scan_order_1 != scan_order_0:
            if ShowJumpOfOut(scan_order_1) == False:
                continue


        if scan_order_2 != scan_order_0 and scan_order_1 != scan_order_2:
            if ShowJumpOfOut(scan_order_2) == False:
                continue


        if scan_order_3 != scan_order_0 and scan_order_3 != scan_order_1 and scan_order_3 != scan_order_2:
            if ShowJumpOfOut(scan_order_3) == False:
                continue

        input("Check Jump and press Enter to continue")

        WriteCommand("tcmscan:4:{0}:{1}:{2}:{3}".format(outIdx[scan_order_0], outIdx[scan_order_1], outIdx[scan_order_2], outIdx[scan_order_3]), "lp5812_chip_setup/dev_config")

        leds = ShowScanDriveModeSelectLed(scan_order_0)
        if leds == None:
            return
        
        if scan_order_1 != scan_order_0:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_1)


        if scan_order_2 != scan_order_0 and scan_order_2 != scan_order_1:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_2)

        if scan_order_3 != scan_order_0 and scan_order_3 != scan_order_1 and scan_order_3 != scan_order_2:
            leds = leds + ShowScanDriveModeSelectLed(scan_order_3)

        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    

def ShowJumpOfOut(scan_out):
    if scan_out == "out_0":
        print("Scan jump config: J3.1 - J3.3")
    elif scan_out == "out_1":
        print("Scan jump config: J11.1 - J11.3")
    elif scan_out == "out_2":
        print("Scan jump config: J27.1 - J27.3")
    elif scan_out == "out_3":
        print("Scan jump config: J31.1 - J31.3")
    else:
        input("Wrong scan order. Press Enter to continue")
        return False
    return True


def ShowMixDriveModeScanJump(direct_output, scan_order):
    if direct_output == "out_0":
        if scan_order == "out_1":
            return "J11.2, J11.3"
        elif scan_order == "out_2":
            return "J27.1, J27.3"
        elif scan_order == "out_3":
            return "J31.1, J31.2"
    elif direct_output == "out_1":
        if scan_order == "out_0":
            return "J3.2, J3.3"
        elif scan_order == "out_2":
            return "J27.0, J27.3"
        elif scan_order == "out_3":
            return "J31.0, J31.2"
    elif direct_output == "out_2":
        if scan_order == "out_0":
            return "J3.1, J3.3"
        elif scan_order == "out_1":
            return "J11.0, J11.3"
        elif scan_order == "out_3":
            return "J31.0, J31.1"
    elif direct_output == "out_3":
        if scan_order == "out_0":
            return "J3.1, J3.2"
        elif scan_order == "out_1":
            return "J11.0, J11.2"
        elif scan_order == "out_2":
            return "J27.0, J27.1"

def ShowMixDriveModeDirectJump(direct_output):
    if direct_output == "out_0":
        return "J1.OUT0, J5.OUT1 - J5.OUT3"
    elif direct_output == "out_1":
        return "J1.OUT1, J5.OUT0, J5.OUT2, J5.OUT3"
    elif direct_output == "out_2":
        return "J1.OUT2, J5.OUT0, J5.OUT1, J5.OUT3"
    elif direct_output == "out_3":
        return "J1.OUT3, J5.OUT0 - J5.OUT2"
    
def ShowMixDriveModeScanSelectLed(direct_output, scan_order):
    selelectled = []
    if direct_output == "out_0":
        if scan_order == "out_1":
            selelectled = ["led_B0", "led_B1"]
        elif scan_order == "out_2":
            selelectled = ["led_C0", "led_C2"]
        elif scan_order == "out_3":
            selelectled = ["led_D0", "led_D2"]
    elif direct_output == "out_1":
        if scan_order == "out_0":
            selelectled = ["led_A1", "led_A2"]
        elif scan_order == "out_2":
            selelectled = ["led_C0", "led_C1"]
        elif scan_order == "out_3":
            selelectled = ["led_D0", "led_D2"]
    elif direct_output == "out_2":
        if scan_order == "out_0":
            selelectled = ["led_A0", "led_A2"]
        elif scan_order == "out_1":
            selelectled = ["led_B1", "led_B2"]
        elif scan_order == "out_3":
            selelectled = ["led_D0", "led_D1"]
    elif direct_output == "out_3":
        if scan_order == "out_0":
            selelectled = ["led_A0", "led_A1"]
        elif scan_order == "out_1":
            selelectled = ["led_B0", "led_B2"]
        elif scan_order == "out_2":
            selelectled = ["led_C1", "led_C2"]

    return selelectled

              

def ShowMixDriveModeWith1Scan():
    print("\033c", end='')
    print("Mix Drive Mode with 1 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36")


def ShowMixDriveMode1ScanSelectLed(direct_output, scan_order_0):
    directLeds = {"out_0": "led_0", "out_1": "led_1", "out_2": "led_2", "out_3": "led_3"}
    selelectled = [directLeds[direct_output]]

    selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_0)

    while True:
        ch = input("Select leds ({0}):".format(', '.join(selelectled)))
        if ch.strip() == "":
            input("Wrong selected led. Press ENTER to continue")
            continue

        led_input = ch.split(",")
        leds = []
        for led in led_input:
            led = led.strip()
            leds.append(led)
        if ValidateLeds(leds, selelectled) == False:
            input("Wrong selected led. Press ENTER to continue")
            continue

        return leds
                    

def MixDriveModeWith1Scan():
    while True:
        ShowMixDriveModeWith1Scan()

        direct_output = input("Direct Connect Output [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if direct_output == "b":
            return

        if not direct_output in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid output. Press Enter to continue")
            continue
        
        
        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_0 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        print("Mix mode jump: " + ",".join([ShowMixDriveModeDirectJump(direct_output), ShowMixDriveModeScanJump(direct_output, scan_order_0)]))

        input("Check jump and press ENTER to continue")
        WriteCommand("mixscan:1:{0}:{1}".format(outIdx[direct_output], outIdx[scan_order_0]), "lp5812_chip_setup/dev_config")


        leds = ShowMixDriveMode1ScanSelectLed(direct_output, scan_order_0)
        if leds == None:
            return
        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    

def ShowMixDriveModeWith2Scan():
    print("\033c", end='')
    print("Mix Drive Mode with 2 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36")


def ShowMixDriveMode2ScanSelectLed(direct_output, scan_order_0, scan_order_1):
    directLeds = {"out_0": "led_0", "out_1": "led_1", "out_2": "led_2", "out_3": "led_3"}
    selelectled = [directLeds[direct_output]]

    selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_0)
    if scan_order_1 != scan_order_0:
        selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_1)

    while True:
        ch = input("Select leds ({0}):".format(', '.join(selelectled)))

        led_input = ch.split(",")
        leds = []
        for led in led_input:
            led = led.strip()
            leds.append(led)
        if ValidateLeds(leds, selelectled) == False:
            input("Wrong selected led. Press ENTER to continue")
            continue

        return leds

def MixDriveModeWith2Scan():
    while True:
        ShowMixDriveModeWith2Scan()

        direct_output = input("Direct Connect Output [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if direct_output == "b":
            return
        
        if not direct_output in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid output. Press Enter to continue")
            continue

        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_0 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue
        scan_order_1 = input("Scan order 1 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_1 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        print("Mix mode jump: " + ",".join([ShowMixDriveModeDirectJump(direct_output), 
                                            ShowMixDriveModeScanJump(direct_output, scan_order_0),
                                            None if scan_order_1 == scan_order_0 else ShowMixDriveModeScanJump(direct_output, scan_order_1),
                                            ]))

        input("Check jump and press ENTER to continue")
        WriteCommand("mixscan:2:{0}:{1}:{2}".format(outIdx[direct_output], outIdx[scan_order_0], outIdx[scan_order_1]), "lp5812_chip_setup/dev_config")

        leds = ShowMixDriveMode2ScanSelectLed(direct_output, scan_order_0, scan_order_1)
        if leds == None:
            return
        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    




def ShowMixDriveModeWith3Scan():
    print("\033c", end='')
    print("Mix Drive Mode with 3 Scan ")
    print("Jump configuration: J12 (VOUT_13 - VLED), J33-J36")


def ShowMixDriveMode3ScanSelectLed(direct_output, scan_order_0, scan_order_1, scan_order_2):
    directLeds = {"out_0": "led_0", "out_1": "led_1", "out_2": "led_2", "out_3": "led_3"}
    selelectled = [directLeds[direct_output]]

    selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_0)
    if scan_order_1 != scan_order_0:
        selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_1)
    if scan_order_2 != scan_order_0 and scan_order_2 != scan_order_1:
        selelectled = selelectled + ShowMixDriveModeScanSelectLed(direct_output, scan_order_2)

    while True:
        ch = input("Select leds ({0}):".format(', '.join(selelectled)))

        led_input = ch.split(",")
        leds = []
        for led in led_input:
            led = led.strip()
            leds.append(led)
        if ValidateLeds(leds, selelectled) == False:
            input("Wrong selected led. Press ENTER to continue")
            continue

        return leds

def MixDriveModeWith3Scan():
    while True:
        ShowMixDriveModeWith3Scan()

        direct_output = input("Direct Connect Output [out_0/out_1/out_2/out_3/b (Back to main menu)]: ")

        if direct_output == "b":
            return
        
        if not direct_output in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid output. Press Enter to continue")
            continue
                                 
        
        scan_order_0 = input("Scan order 0 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_0 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        scan_order_1 = input("Scan order 1 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_1 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        scan_order_2 = input("Scan order 2 [out_0/out_1/out_2/out_3]: ")
        if not scan_order_2 in ["out_0", "out_1", "out_2", "out_3"]:
            input("Select invalid scan output. Press Enter to continue")
            continue

        print("Mix mode jump: " + ",".join([ShowMixDriveModeDirectJump(direct_output), 
                                            ShowMixDriveModeScanJump(direct_output, scan_order_0),
                                            None if scan_order_1 == scan_order_0 else ShowMixDriveModeScanJump(direct_output, scan_order_1),
                                            None if scan_order_2 == scan_order_0 and scan_order_2 == scan_order_1 else ShowMixDriveModeScanJump(direct_output, scan_order_2),
                                            ]))
        
        input("Check jump and press ENTER to continue")
        WriteCommand("mixscan:3:{0}:{1}:{2}:{3}".format(outIdx[direct_output], outIdx[scan_order_0], outIdx[scan_order_1], outIdx[scan_order_2]), "lp5812_chip_setup/dev_config")


        leds = ShowMixDriveMode3ScanSelectLed(direct_output, scan_order_0, scan_order_1, scan_order_2)
        if leds == None:
            return
        print("You selected : " + ', '.join(leds))

        if PerformDriveLeds(leds) == False:
            return    
        
def EnableDevice():
    ch = input("Enable Device (1. enable/0. disable): ")
    if ch == "1":
        WriteCommand("1", "lp5812_chip_setup/device_enable")
    else:
        WriteCommand("0", "lp5812_chip_setup/device_enable")

    input("Press ENTER to continue")

def ResetDevice():
    WriteCommand("1", "lp5812_chip_setup/device_reset")
    input("Press ENTER to continue")    

def ResetFault():
    ch = input("Clear value (0. Clear LOD/ 1. Clear LSD / 2. Clear TSD / 3. Clear all  ): ")
    WriteCommand(ch, "lp5812_chip_setup/fault_clear")
    input("Press ENTER to continue")  

def ShowLedDriveModeSelection():
    print("\033c", end='')
    print("1. Direct Drive Mode")
    print("2. Scan Drive Mode with 1 Scan")
    print("3. Scan Drive Mode with 2 Scans")
    print("4. Scan Drive Mode with 3 Scans")
    print("5. Scan Drive Mode with 4 Scans")
    print("6. Mix Drive Mode with 1 Scan")
    print("7. Mix Drive Mode with 2 Scans")
    print("8. Mix Drive Mode with 3 Scans")
    print("9. Enable device")
    print("10. Reset device")
    print("11. Clear fault")
    print("0. Exit")
    print("")

  
def LedDriveModeSelection():

    while True:
        ShowLedDriveModeSelection()
        ch = input("Select from 0-11 to continue: ")

        if ch == "0":
            exit()
        elif  ch == "1":
            DirectDriveMode()
        elif  ch == "2":
            ScanDriveModeWith1Scan()
        elif  ch == "3":
            ScanDriveModeWith2Scan()
        elif  ch == "4":
            ScanDriveModeWith3Scan()
        elif  ch == "5":
            ScanDriveModeWith4Scan()
        elif  ch == "6":
            MixDriveModeWith1Scan()
        elif  ch == "7":
            MixDriveModeWith2Scan()
        elif  ch == "8":
            MixDriveModeWith3Scan()
        elif  ch == "9":
            EnableDevice()
        elif  ch == "10":
            ResetDevice()
        elif  ch == "11":
            ResetFault()


def Init():
    if not os.path.exists(driverPath):
        print("Cannot found LP5812 driver. Exit...")
        exit()

Init()
LedDriveModeSelection()
